!***************************************************
!*
!* Module for the preconditionning of linear system
!*
!*                    A X = B
!*
!***************************************************

MODULE preconditionning

  !* Loading needed module

  USE constants

  CONTAINS

    !**************************************************
    !*
    !* Subroutine applying the Jacobi preconditionning
    !* also named diagonal preconditionning
    !*
    !* We compute : 
    !*
    !*           (C^-1)A X = (C^-1)B
    !*
    !* With C=diag(A)
    !*
    !**************************************************

    subroutine PRECONDITIONNING(a,b)

    IMPLICIT NONE

    !* Variables declaration

    REAL(idp), DIMENSION(:,:)                 :: a
    REAL(idp), DIMENSION(size(a,1),size(a,1)) :: tempMat,ident
    REAL(idp), DIMENSION(:)                   :: b
    REAL(idp), DIMENSION(size(a,1))           :: tempVec
    INTEGER                                   :: i

    !* Code

    !** Initialization of the preconditionning matrix

    ident = 0.d0
    
    do i=1,size(a,1)
       ident(i,i) = 1.d0/a(i,i)
    end do

    tempMat = 0.d0
    tempVec = 0.d0

    !** Computation of the new matrix and right hand side

    call dgemm('n','n',size(a,1),size(a,1),size(a,1),1.d0, ident, size(a,1), a, size(a,1), 0.d0,tempMat, size(a,1) )

    call dgemv('n',size(a,1), size(a,1), 1.d0, ident,size(a,1),b,1, 1.d0, tempVec, 1 )

    a = tempMat
    
    b = tempVec

  END subroutine PRECONDITIONNING

END MODULE preconditionning
